CrushChance = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0","AceDB-2.0","AceEvent-2.0")

CrushChance:RegisterDB("CrushChanceDB")
CrushChance:RegisterDefaults("profile", {
	x=400,
	y=400,
	defaultLevel=73
} )

function CrushChance:OnInitialize()
    -- Called when the addon is loaded
	--create frame
	local options = { 
		type='group',
		args = {
			reset = {
				type = 'execute',
				name = 'Reset Position',
				desc = 'Reset the position of crush chance in case you lost it',
				func = 'ResetPosition'
			},
			info = {
				type = 'execute',
				name = 'Calculation Info',
				desc = 'Prints information on how the calculation is performed.',
				func = 'PrintCalcInfo'
			},
			dl = {
				type = 'range',
				name = 'Default Level',
				desc = 'Set the default level to use when you have no target. 0 to use the last target\'s level',
				get = 'GetDefaultLevel',
				set = 'SetDefaultLevel',
				min = 0,
				max = 73,
				step = 1
			}																		    			
		}}
		
	self:RegisterChatCommand({"/crushchance"}, options)    	
end

function CrushChance:OnEnable()
	if self.frame == nil then
		--create frame
		local f = CreateFrame("Frame",nil,UIParent)
		f:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT",
					self.db.profile.x,self.db.profile.y)
		f:SetWidth(60)
		f:SetHeight(20)
		
		--Chance Text
		local t = f:CreateFontString(nil,"OVERLAY","GameFontNormal")
		t:SetAllPoints(f)
		t:SetJustifyH("LEFT")
		f.chanceText = t
		
		--Dual-Wield Texture
		local tex = f:CreateTexture()
		tex:SetTexture("Interface\\Buttons\\UI-Checkbox-SwordCheck")
		tex:SetTexCoord(0,0.593,0,0.5)
		tex:SetPoint("RIGHT",t,"LEFT")
		tex:SetWidth(12)
		tex:SetHeight(10)
		tex:Hide()
		f.dualWieldTex = tex
		
		f:EnableMouse(true)
		f:SetMovable(true)		
		
		f:SetScript("OnMouseDown",
			function()
				if arg1 == "RightButton" or IsModifierKeyDown() then
					f:StartMoving()
				else
					CrushChance:ToggleDualWield()
				end
			end)
		f:SetScript("OnMouseUp",
			function()
				f:StopMovingOrSizing()
				CrushChance:SavePosition()
			end)
			
		self.frame = f
	end
	
	self.dw = false
	self.lastTargetLevel = 70
	self.needNum = 100
	
	self:CalculateTdn()
	self:PLAYER_TARGET_CHANGED()
	self.frame:Show()
	
	self:RegisterEvent("UNIT_AURA","UpdateChance")
	self:RegisterEvent("UNIT_INVENTORY_CHANGED","UpdateChance")
	self:RegisterEvent("CHARACTER_POINTS_CHANGED","CalculateTdn")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
end

function CrushChance:OnDisable()
    -- Called when the addon is disabled
	--self:UnregisterAllBucketEvents()
	self:UnregisterAllEvents()
	self.frame:Hide()
end

function CrushChance:UpdateChance(unitId)
	if unitId~="player" then
		return
	end
	
	local chance = self.needNum - ( GetDodgeChance()+GetBlockChance() + GetParryChance() +5+(GetCombatRatingBonus(CR_DEFENSE_SKILL) + self.tdn)*0.04 )
	if self.dw then
		chance = chance-20
	end

	local colorStr
	if chance<=0 then
		chance=0
		--Can't be hit (green)
		colorStr="|cFF00FF00"
	elseif chance <= 20 then
		--crushes only (orange)
		colorStr = "|cFFFF8C2F"
	else
		--hits and crushes (red)
		colorStr="|cFFFF0000"
	end
	
	self.frame.chanceText:SetText( string.format("%s%.1f|r",colorStr, chance) )
end

function CrushChance:CalculateTdn()
	local _,playerClass = UnitClass("player")
	if playerClass == 'WARRIOR' then		
		self.tdn = 4 * select(5, GetTalentInfo(3, 3))
	elseif playerClass=='PALADIN' then
		self.tdn = 4 * select(5, GetTalentInfo(2, 9))
	else
		self.tdn = 0			
	end
	
	self:Print(string.format("Your defense contribution from talents is %d, this should not be zero if you are a warrior or paladin with Anticipation.",self.tdn))
	self:UpdateChance("player")
end

function CrushChance:SavePosition()
	self.db.profile.x = self.frame:GetLeft()
	self.db.profile.y = self.frame:GetTop()
end	

function CrushChance:PLAYER_TARGET_CHANGED()
	--0.8 per level above you 3*0.8 = 2.4
	local targetLevel = UnitLevel("target")
	local defaultLevel = self.db.profile.defaultLevel
	
	if targetLevel <= 0 then
		--no target or unknown
		if defaultLevel == 0 then
			targetLevel = self.lastTargetLevel
		else
			targetLevel = defaultLevel
		end
	else
		self.lastTargetLevel = targetLevel
	end
	
	self.needNum = 100 + (targetLevel-UnitLevel("player"))*0.8
	
	self:UpdateChance("player")
end

function CrushChance:ToggleDualWield()
	if self.dw then
		self.dw = false
		self.frame.dualWieldTex:Hide()
	else
		self.dw = true
		self.frame.dualWieldTex:Show()
	end
	self:UpdateChance("player")
end

--Executes
function CrushChance:ResetPosition()
	local f = self.frame
	f:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT",
		UIParent:GetWidth()/2, UIParent:GetHeight()/2)	
end
function CrushChance:PrintCalcInfo()
	self:Print("The number you see is calculated with the following formula:\n(100+LevelDifference*0.8) - ( GetDodgeChance()+GetBlockChance() + GetParryChance() +5+(GetCombatRatingBonus(CR_DEFENSE_SKILL) + TDN)*0.04 )\n\nThis represents your chance to be hit or crushed, if the number is below 20 any hits you receive will be crushing.")
end

-- GET/SET
function CrushChance:SetDefaultLevel(v)
	self.db.profile.defaultLevel = v
	self:PLAYER_TARGET_CHANGED()
end

function CrushChance:GetDefaultLevel()
	return self.db.profile.defaultLevel
end

